@@ -3,9 +3,13 @@
 #[cfg(feature = "default")]
 mod cli;
 
-use std::{fs, path::Path, process::Command, str};
-
-use tempfile::Builder;
+use std::{
+    env, fs,
+    io::Write,
+    path::Path,
+    process::{Command, Stdio},
+    str,
+};
 
 #[cfg(feature = "default")]
 pub use self::cli::*;
@@ -27,18 +31,21 @@ pub fn trim(s: &str) -> &str {
 pub fn assert_diff(expected_path: impl AsRef<Path>, actual: impl AsRef<str>) {
     let actual = actual.as_ref();
     let manifest_dir = Path::new(env!("CARGO_MANIFEST_DIR"));
+    let manifest_dir =
+        manifest_dir.strip_prefix(env::current_dir().unwrap()).unwrap_or(manifest_dir);
     let expected_path = &manifest_dir.join(expected_path);
     let expected = fs::read_to_string(expected_path).unwrap();
     if expected.trim() != actual.trim() {
-        let outdir = Builder::new().prefix("assert_diff").tempdir().unwrap();
-        let actual_path = &outdir.path().join(expected_path.file_name().unwrap());
-        fs::write(actual_path, actual).unwrap();
-        let status = Command::new("git")
-            .args(&["--no-pager", "diff", "--no-index", "--"])
-            .args(&[expected_path, actual_path])
-            .status()
+        let mut child = Command::new("git")
+            .args(["--no-pager", "diff", "--no-index", "--"])
+            .arg(expected_path)
+            .arg("-")
+            .stdin(Stdio::piped())
+            .spawn()
             .unwrap();
-        assert!(!status.success());
-        panic!("assertion failed");
+        child.stdin.as_mut().unwrap().write_all(actual.as_bytes()).unwrap();
+        assert!(!child.wait().unwrap().success());
+        // patch -p1 <<'EOF' ... EOF
+        panic!("assertion failed; please run test locally and commit resulting changes, or apply above diff as patch");
     }
 }
